@@ -384,6 +384,16 @@ Builder = override(Builder)
 __all__.append('Builder')
 
 
+# NOTE: This must come before any other Window/Dialog subclassing, to ensure
+# that we have a correct inheritance hierarchy.
+class Window(Gtk.Window):
+    def __init__(self, type=Gtk.WindowType.TOPLEVEL, **kwargs):
+        Gtk.Window.__init__(self, type=type, **kwargs)
+
+Window = override(Window)
+__all__.append('Window')
+
+
 class Dialog(Gtk.Dialog, Container):
 
     def __init__(self,
@@ -470,8 +480,9 @@ class MessageDialog(Gtk.MessageDialog, Dialog):
         Gtk.MessageDialog.__init__(self,
                                    _buttons_property=buttons,
                                    message_type=message_type,
+                                   parent=parent,
+                                   flags=flags,
                                    **kwds)
-        Dialog.__init__(self, parent=parent, flags=flags)
 
     def format_secondary_text(self, message_format):
         self.set_property('secondary-use-markup', False)
@@ -484,23 +495,21 @@ class MessageDialog(Gtk.MessageDialog, Dialog):
 MessageDialog = override(MessageDialog)
 __all__.append('MessageDialog')
 
-class AboutDialog(Gtk.AboutDialog, Dialog):
+class AboutDialog(Gtk.AboutDialog):
     def __init__(self, **kwds):
         Gtk.AboutDialog.__init__(self, **kwds)
-        Dialog.__init__(self)
 
 AboutDialog = override(AboutDialog)
 __all__.append('AboutDialog')
 
-class ColorSelectionDialog(Gtk.ColorSelectionDialog, Dialog):
+class ColorSelectionDialog(Gtk.ColorSelectionDialog):
     def __init__(self, title=None, **kwds):
-        Gtk.ColorSelectionDialog.__init__(self, **kwds)
-        Dialog.__init__(self, title=title)
+        Gtk.ColorSelectionDialog.__init__(self, title=title, **kwds)
 
 ColorSelectionDialog = override(ColorSelectionDialog)
 __all__.append('ColorSelectionDialog')
 
-class FileChooserDialog(Gtk.FileChooserDialog, Dialog):
+class FileChooserDialog(Gtk.FileChooserDialog):
     def __init__(self,
                  title=None,
                  parent=None,
@@ -509,24 +518,21 @@ class FileChooserDialog(Gtk.FileChooserDialog, Dialog):
                  **kwds):
         Gtk.FileChooserDialog.__init__(self,
                                        action=action,
+                                       title=title,
+                                       parent=parent,
+                                       buttons=buttons,
                                        **kwds)
-        Dialog.__init__(self,
-                        title=title,
-                        parent=parent,
-                        buttons=buttons)
-
 FileChooserDialog = override(FileChooserDialog)
 __all__.append('FileChooserDialog')
 
-class FontSelectionDialog(Gtk.FontSelectionDialog, Dialog):
+class FontSelectionDialog(Gtk.FontSelectionDialog):
     def __init__(self, title=None, **kwds):
-        Gtk.FontSelectionDialog.__init__(self, **kwds)
-        Dialog.__init__(self, title=title)
+        Gtk.FontSelectionDialog.__init__(self, title=title, **kwds)
 
 FontSelectionDialog = override(FontSelectionDialog)
 __all__.append('FontSelectionDialog')
 
-class RecentChooserDialog(Gtk.RecentChooserDialog, Dialog):
+class RecentChooserDialog(Gtk.RecentChooserDialog):
     def __init__(self,
                  title=None,
                  parent=None,
@@ -534,17 +540,21 @@ class RecentChooserDialog(Gtk.RecentChooserDialog, Dialog):
                  buttons=None,
                  **kwds):
 
-        Gtk.RecentChooserDialog.__init__(self, recent_manager=manager, **kwds)
-        Dialog.__init__(self,
-                        title=title,
-                        parent=parent,
-                        buttons=buttons)
+        Gtk.RecentChooserDialog.__init__(self,
+                recent_manager=manager,
+                title=title,
+                parent=parent,
+                buttons=buttons,
+                **kwds)
 
 RecentChooserDialog = override(RecentChooserDialog)
 __all__.append('RecentChooserDialog')
 
 class IconView(Gtk.IconView):
 
+    def __init__(self, model=None):
+        Gtk.IconView.__init__(self, model=model)
+
     def get_item_at_pos(self, x, y):
         success, path, cell = super(IconView, self).get_item_at_pos(x, y)
         if success:
@@ -563,6 +573,16 @@ class IconView(Gtk.IconView):
 IconView = override(IconView)
 __all__.append('IconView')
 
+
+class ToolButton(Gtk.ToolButton):
+
+    def __init__(self, stock_id=None):
+        Gtk.ToolButton.__init__(self, stock_id=stock_id)
+
+ToolButton = override(ToolButton)
+__all__.append('ToolButton')
+
+
 class IMContext(Gtk.IMContext):
 
     def get_surrounding(self):
@@ -778,7 +798,7 @@ class TreeModel(Gtk.TreeModel):
         if success:
             return parent_iter
 
-    def set_row(self, treeiter, row):
+    def _convert_row(self, row):
         # TODO: Accept a dictionary for row
         # model.append(None,{COLUMN_ICON: icon, COLUMN_NAME: name})
         if isinstance(row, str):
@@ -788,16 +808,24 @@ class TreeModel(Gtk.TreeModel):
         if len(row) != n_columns:
             raise ValueError('row sequence has the incorrect number of elements')
 
+        result = []
         for i in range(n_columns):
+            value = row[i]
+            result.append(self._convert_value(i, value))
+        return result
+
+    def set_row(self, treeiter, row):
+        converted_row = self._convert_row(row)
+        for i in range(self.get_n_columns()):
             value = row[i]
             if value is None:
                continue  # None means skip this row
 
             self.set_value(treeiter, i, value)
 
-    def _convert_value(self, treeiter, column, value):
+    def _convert_value(self, column, value):
             if value is None:
-                return
+                return None
 
             # we may need to convert to a basic type
             type_ = self.get_column_type(column)
@@ -919,22 +947,32 @@ class ListStore(Gtk.ListStore, TreeModel, TreeSortable):
         Gtk.ListStore.__init__(self)
         self.set_column_types(column_types)
 
-    def append(self, row=None):
-        treeiter = Gtk.ListStore.append(self)
-
+    def _do_insert(self, position, row):
         if row is not None:
-            self.set_row(treeiter, row)
+            row = self._convert_row(row)
+            columns = range(len(row))
+            treeiter = self.insert_with_valuesv(position, columns, row)
+        else:
+            treeiter = Gtk.ListStore.insert(self, position)
 
         return treeiter
 
+    def append(self, row=None):
+        if row:
+            return self._do_insert(-1, row)
+        # gtk_list_store_insert() does not know about the "position == -1"
+        # case, so use append() here
+        else:
+            return Gtk.ListStore.append(self)
+
+    def prepend(self, row=None):
+        return self._do_insert(0, row)
+
     def insert(self, position, row=None):
-        treeiter = Gtk.ListStore.insert(self, position)
-
-        if row is not None:
-            self.set_row(treeiter, row)
-
-        return treeiter
+        return self._do_insert(position, row)
 
+    # FIXME: sends two signals; check if this can use an atomic
+    # insert_with_valuesv()
     def insert_before(self, sibling, row=None):
         treeiter = Gtk.ListStore.insert_before(self, sibling)
 
@@ -943,6 +981,8 @@ class ListStore(Gtk.ListStore, TreeModel, TreeSortable):
 
         return treeiter
 
+    # FIXME: sends two signals; check if this can use an atomic
+    # insert_with_valuesv()
     def insert_after(self, sibling, row=None):
         treeiter = Gtk.ListStore.insert_after(self, sibling)
 
@@ -951,16 +991,8 @@ class ListStore(Gtk.ListStore, TreeModel, TreeSortable):
 
         return treeiter
 
-    def prepend(self, row=None):
-        treeiter = Gtk.ListStore.prepend(self)
-
-        if row is not None:
-            self.set_row(treeiter, row)
-
-        return treeiter
-
     def set_value(self, treeiter, column, value):
-        value = self._convert_value(treeiter, column, value)
+        value = self._convert_value(column, value)
         Gtk.ListStore.set_value(self, treeiter, column, value)
 
     def set(self, treeiter, *args):
@@ -1133,6 +1165,9 @@ class TreePath(Gtk.TreePath):
     def __ge__(self, other):
         return other is None or self.compare(other) >= 0
 
+    def __iter__(self):
+        return (int(part) for part in str(self).split(':'))
+
 TreePath = override(TreePath)
 __all__.append('TreePath')
 
@@ -1142,22 +1177,27 @@ class TreeStore(Gtk.TreeStore, TreeModel, TreeSortable):
         Gtk.TreeStore.__init__(self)
         self.set_column_types(column_types)
 
-    def append(self, parent, row=None):
-        treeiter = Gtk.TreeStore.append(self, parent)
-
+    def _do_insert(self, parent, position, row):
         if row is not None:
-            self.set_row(treeiter, row)
+            row = self._convert_row(row)
+            columns = range(len(row))
+            treeiter = self.insert_with_values(parent, position, columns, row)
+        else:
+            treeiter = Gtk.TreeStore.insert(self, parent, position)
 
         return treeiter
 
+    def append(self, parent, row=None):
+        return self._do_insert(parent, -1, row)
+
+    def prepend(self, parent, row=None):
+        return self._do_insert(parent, 0, row)
+
     def insert(self, parent, position, row=None):
-        treeiter = Gtk.TreeStore.insert(self, parent, position)
-
-        if row is not None:
-            self.set_row(treeiter, row)
-
-        return treeiter
+        return self._do_insert(parent, position, row)
 
+    # FIXME: sends two signals; check if this can use an atomic
+    # insert_with_valuesv()
     def insert_before(self, parent, sibling, row=None):
         treeiter = Gtk.TreeStore.insert_before(self, parent, sibling)
 
@@ -1166,6 +1206,8 @@ class TreeStore(Gtk.TreeStore, TreeModel, TreeSortable):
 
         return treeiter
 
+    # FIXME: sends two signals; check if this can use an atomic
+    # insert_with_valuesv()
     def insert_after(self, parent, sibling, row=None):
         treeiter = Gtk.TreeStore.insert_after(self, parent, sibling)
 
@@ -1175,7 +1217,7 @@ class TreeStore(Gtk.TreeStore, TreeModel, TreeSortable):
         return treeiter
 
     def set_value(self, treeiter, column, value):
-        value = self._convert_value(treeiter, column, value)
+        value = self._convert_value(column, value)
         Gtk.TreeStore.set_value(self, treeiter, column, value)
 
     def set(self, treeiter, *args):
@@ -1219,6 +1261,11 @@ class TreeView(Gtk.TreeView, Container):
         if success:
             return (path, column, cell_x, cell_y,)
 
+    def get_visible_range(self):
+        success, start_path, end_path = super(TreeView, self).get_visible_range()
+        if success:
+            return (start_path, end_path,)
+
     def get_dest_row_at_pos(self, drag_x, drag_y):
         success, path, pos = super(TreeView, self).get_dest_row_at_pos(drag_x, drag_y)
         if success:
@@ -1249,6 +1296,15 @@ class TreeView(Gtk.TreeView, Container):
             path = TreePath(path)
         super(TreeView, self).scroll_to_cell(path, column, use_align, row_align, col_align)
 
+    def set_cursor(self, path, column=None, start_editing=False):
+        if not isinstance(path, Gtk.TreePath):
+            path = TreePath(path)
+        super(TreeView, self).set_cursor(path, column, start_editing)
+
+    def get_cell_area(self, path, column=None):
+        if not isinstance(path, Gtk.TreePath):
+            path = TreePath(path)
+        return super(TreeView, self).get_cell_area(path, column)
 
 TreeView = override(TreeView)
 __all__.append('TreeView')
@@ -1299,13 +1355,11 @@ TreeSelection = override(TreeSelection)
 __all__.append('TreeSelection')
 
 class Button(Gtk.Button, Container):
-    def __init__(self, label=None, stock=None, use_underline=False, **kwds):
+    def __init__(self, label=None, stock=None, use_stock=False, use_underline=False, **kwds):
         if stock:
             label = stock
             use_stock = True
             use_underline = True
-        else:
-            use_stock = False
         Gtk.Button.__init__(self, label=label, use_stock=use_stock,
                             use_underline=use_underline, **kwds)
 Button = override(Button)
@@ -1328,9 +1382,15 @@ __all__.append('Label')
 class Adjustment(Gtk.Adjustment):
     def __init__(self, *args, **kwds):
         arg_names = ('value', 'lower', 'upper',
-                        'step_increment', 'page_increment', 'page_size')
+                     'step_increment', 'page_increment', 'page_size')
         new_args = dict(zip(arg_names, args))
         new_args.update(kwds)
+
+        # PyGTK compatiblity
+        if 'page_incr' in new_args:
+            new_args['page_increment'] = new_args.pop('page_incr')
+        if 'step_incr' in new_args:
+            new_args['step_increment'] = new_args.pop('step_incr')
         Gtk.Adjustment.__init__(self, **new_args)
 
         # The value property is set between lower and (upper - page_size).
@@ -1365,6 +1425,21 @@ class ScrolledWindow(Gtk.ScrolledWindow):
 ScrolledWindow = override(ScrolledWindow)
 __all__.append('ScrolledWindow')
 
+
+class HScrollbar(Gtk.HScrollbar):
+    def __init__(self, adjustment=None):
+        Gtk.HScrollbar.__init__(self, adjustment=adjustment)
+
+HScrollbar = override(HScrollbar)
+__all__.append('HScrollbar')
+
+class VScrollbar(Gtk.VScrollbar):
+    def __init__(self, adjustment=None):
+        Gtk.VScrollbar.__init__(self, adjustment=adjustment)
+
+VScrollbar = override(VScrollbar)
+__all__.append('VScrollbar')
+
 class Paned(Gtk.Paned):
     def pack1(self, child, resize=False, shrink=True):
         super(Paned, self).pack1(child, resize, shrink)
@@ -1375,6 +1450,44 @@ class Paned(Gtk.Paned):
 Paned = override(Paned)
 __all__.append('Paned')
 
+
+class Arrow(Gtk.Arrow):
+    def __init__(self, arrow_type, shadow_type):
+        Gtk.Arrow.__init__(self, arrow_type=arrow_type,
+                           shadow_type=shadow_type)
+
+Arrow = override(Arrow)
+__all__.append('Arrow')
+
+
+class IconSet(Gtk.IconSet):
+    def __new__(cls, pixbuf=None):
+        if pixbuf is not None:
+            iconset = Gtk.IconSet.new_from_pixbuf(pixbuf)
+        else:
+            iconset = Gtk.IconSet.__new__(cls)
+        return iconset
+
+IconSet = override(IconSet)
+__all__.append('IconSet')
+
+
+class Viewport(Gtk.Viewport):
+    def __init__(self, hadjustment=None, vadjustment=None):
+        Gtk.Viewport.__init__(self, hadjustment=hadjustment,
+                              vadjustment=vadjustment)
+
+Viewport = override(Viewport)
+__all__.append('Viewport')
+
+
+class TreeModelFilter(Gtk.TreeModelFilter):
+    def set_visible_func(self, func, data=None):
+        super(TreeModelFilter, self).set_visible_func(func, data)
+
+TreeModelFilter = override(TreeModelFilter)
+__all__.append('TreeModelFilter')
+
 if Gtk._version != '2.0':
     class Menu(Gtk.Menu):
         def popup(self, parent_menu_shell, parent_menu_item, func, data, button, activate_time):
