@@ -61,21 +61,18 @@ _is_union_member (PyGIInterfaceCache *iface_cache, PyObject *py_arg) {
 
         /* we can only check if the members are interfaces */
         if (g_type_info_get_tag (field_type_info) == GI_TYPE_TAG_INTERFACE) {
+            GIInterfaceInfo *field_iface_info;
             PyObject *py_type;
-            GIInterfaceInfo *field_iface_info =
-                g_type_info_get_interface (field_type_info);
 
-            py_type = _pygi_type_import_by_gi_info (
-                (GIBaseInfo *) field_iface_info);
+            field_iface_info = g_type_info_get_interface (field_type_info);
+            py_type = _pygi_type_import_by_gi_info ((GIBaseInfo *) field_iface_info);
 
             if (py_type != NULL && PyObject_IsInstance (py_arg, py_type)) {
                 is_member = TRUE;
-                break;
             }
 
             Py_XDECREF (py_type);
             g_base_info_unref ( ( GIBaseInfo *) field_iface_info);
-
         }
 
         g_base_info_unref ( ( GIBaseInfo *) field_type_info);
@@ -148,7 +145,7 @@ _pygi_marshal_from_py_int8 (PyGIInvokeState   *state,
         return FALSE;
     }
 
-    arg->v_long = long_;
+    arg->v_int8 = long_;
 
     return TRUE;
 }
@@ -197,7 +194,7 @@ _pygi_marshal_from_py_uint8 (PyGIInvokeState   *state,
         return FALSE;
     }
 
-    arg->v_long = long_;
+    arg->v_uint8 = long_;
 
     return TRUE;
 }
@@ -236,7 +233,7 @@ _pygi_marshal_from_py_int16 (PyGIInvokeState   *state,
         return FALSE;
     }
 
-    arg->v_long = long_;
+    arg->v_int16 = long_;
 
     return TRUE;
 }
@@ -275,7 +272,7 @@ _pygi_marshal_from_py_uint16 (PyGIInvokeState   *state,
         return FALSE;
     }
 
-    arg->v_long = long_;
+    arg->v_uint16 = long_;
 
     return TRUE;
 }
@@ -314,7 +311,7 @@ _pygi_marshal_from_py_int32 (PyGIInvokeState   *state,
         return FALSE;
     }
 
-    arg->v_long = long_;
+    arg->v_int32 = long_;
 
     return TRUE;
 }
@@ -359,7 +356,7 @@ _pygi_marshal_from_py_uint32 (PyGIInvokeState   *state,
         return FALSE;
     }
 
-    arg->v_uint64 = long_;
+    arg->v_uint32 = long_;
 
     return TRUE;
 }
@@ -842,8 +839,24 @@ _pygi_marshal_from_py_array (PyGIInvokeState   *state,
                          * since GVariants are opaque hence always passed by ref */
                         g_assert (item_size == sizeof (item.v_pointer));
                         g_array_insert_val (array_, i, item.v_pointer);
-                    } else if (!is_boxed || is_gvalue) {
+                    } else if (is_gvalue) {
+                        GValue* dest = (GValue*) (array_->data + (i * item_size));
+                        memset (dest, 0, item_size);
+                        if (item.v_pointer != NULL) {
+                            g_value_init (dest, G_VALUE_TYPE ((GValue*) item.v_pointer));
+                            g_value_copy ((GValue*) item.v_pointer, dest);
+                        }
+
+                        if (from_py_cleanup) {
+                            from_py_cleanup (state, item_arg_cache, item.v_pointer, TRUE);
+                            /* we freed the original copy already, the new one is a 
+                             * struct in an array. _pygi_marshal_cleanup_from_py_array()
+                             * must not free it again */
+                            item_arg_cache->from_py_cleanup = NULL;
+                        }
+                    } else if (!is_boxed) {
                         memcpy (array_->data + (i * item_size), item.v_pointer, item_size);
+
                         if (from_py_cleanup)
                             from_py_cleanup (state, item_arg_cache, item.v_pointer, TRUE);
                     } else {
@@ -1044,6 +1057,22 @@ err:
     return TRUE;
 }
 
+static gpointer
+_pygi_arg_to_hash_pointer (const GIArgument *arg,
+                           GITypeTag        type_tag)
+{
+    switch (type_tag) {
+        case GI_TYPE_TAG_INT32:
+            return GINT_TO_POINTER(arg->v_int32);
+        case GI_TYPE_TAG_UTF8:
+        case GI_TYPE_TAG_FILENAME:
+            return arg->v_pointer;
+        default:
+            g_assert_not_reached();
+            return arg->v_pointer;
+    }
+}
+
 gboolean
 _pygi_marshal_from_py_ghash (PyGIInvokeState   *state,
                              PyGICallableCache *callable_cache,
@@ -1131,7 +1160,9 @@ _pygi_marshal_from_py_ghash (PyGIInvokeState   *state,
                                        &value))
             goto err;
 
-        g_hash_table_insert (hash_, key.v_pointer, value.v_pointer);
+        g_hash_table_insert (hash_,
+                             _pygi_arg_to_hash_pointer (&key, hash_cache->key_cache->type_tag),
+                             _pygi_arg_to_hash_pointer (&value, hash_cache->value_cache->type_tag));
         continue;
 err:
         /* FIXME: cleanup hash keys and values */
