@@ -465,6 +465,10 @@ class TestPango(unittest.TestCase):
         layout = Pango.Layout(context)
         self.assertEquals(layout.get_context(), context)
 
+        layout.set_markup("Foobar")
+        self.assertEquals(layout.get_text(), "Foobar")
+
+
 class TestGdk(unittest.TestCase):
 
     def test_constructor(self):
@@ -558,6 +562,9 @@ class TestGtk(unittest.TestCase):
 
     def test_container(self):
         box = Gtk.Box()
+        self.failUnless(isinstance(box, Gtk.Box))
+        self.failUnless(isinstance(box, Gtk.Container))
+        self.failUnless(isinstance(box, Gtk.Widget))
         self.assertTrue(box)
         label = Gtk.Label()
         label2 = Gtk.Label()
@@ -736,6 +743,8 @@ class TestGtk(unittest.TestCase):
         dialog = Gtk.Dialog (title='Foo',
                              flags=Gtk.DialogFlags.MODAL,
                              buttons=('test-button1', 1))
+        self.failUnless(isinstance(dialog, Gtk.Dialog))
+        self.failUnless(isinstance(dialog, Gtk.Window))
 
         dialog.add_buttons ('test-button2', 2, Gtk.STOCK_CLOSE, Gtk.ResponseType.CLOSE)
 
@@ -750,12 +759,16 @@ class TestGtk(unittest.TestCase):
 
         # Gtk.AboutDialog
         dialog = Gtk.AboutDialog()
+        self.failUnless(isinstance(dialog, Gtk.Dialog))
+        self.failUnless(isinstance(dialog, Gtk.Window))
 
         # Gtk.MessageDialog
         dialog = Gtk.MessageDialog (title='message dialog test',
                                     flags=Gtk.DialogFlags.MODAL,
                                     buttons=Gtk.ButtonsType.OK,
                                     message_format='dude!')
+        self.failUnless(isinstance(dialog, Gtk.Dialog))
+        self.failUnless(isinstance(dialog, Gtk.Window))
 
         self.assertEquals('message dialog test', dialog.get_title())
         self.assertTrue(dialog.get_modal())
@@ -772,12 +785,16 @@ class TestGtk(unittest.TestCase):
 
         # Gtk.ColorSelectionDialog
         dialog = Gtk.ColorSelectionDialog("color selection dialog test")
+        self.failUnless(isinstance(dialog, Gtk.Dialog))
+        self.failUnless(isinstance(dialog, Gtk.Window))
         self.assertEquals('color selection dialog test', dialog.get_title())
 
         # Gtk.FileChooserDialog
         dialog = Gtk.FileChooserDialog (title='file chooser dialog test',
                                         buttons=('test-button1', 1),
                                         action=Gtk.FileChooserAction.SAVE)
+        self.failUnless(isinstance(dialog, Gtk.Dialog))
+        self.failUnless(isinstance(dialog, Gtk.Window))
 
         dialog.add_buttons ('test-button2', 2, Gtk.STOCK_CLOSE, Gtk.ResponseType.CLOSE)
         self.assertEquals('file chooser dialog test', dialog.get_title())
@@ -793,6 +810,8 @@ class TestGtk(unittest.TestCase):
 
         # Gtk.FontSelectionDialog
         dialog = Gtk.ColorSelectionDialog("font selection dialog test")
+        self.failUnless(isinstance(dialog, Gtk.Dialog))
+        self.failUnless(isinstance(dialog, Gtk.Window))
         self.assertEquals('font selection dialog test', dialog.get_title())
 
         # Gtk.RecentChooserDialog
@@ -800,6 +819,8 @@ class TestGtk(unittest.TestCase):
         dialog = Gtk.RecentChooserDialog (title='recent chooser dialog test',
                                           buttons=('test-button1', 1),
                                           manager=test_manager)
+        self.failUnless(isinstance(dialog, Gtk.Dialog))
+        self.failUnless(isinstance(dialog, Gtk.Window))
 
         dialog.add_buttons ('test-button2', 2, Gtk.STOCK_CLOSE, Gtk.ResponseType.CLOSE)
         self.assertEquals('recent chooser dialog test', dialog.get_title())
@@ -982,6 +1003,43 @@ class TestGtk(unittest.TestCase):
 
         self.assertEquals(i, 99)
 
+    def test_tree_store_signals(self):
+        tree_store = Gtk.TreeStore(int, bool)
+
+        def on_row_inserted(tree_store, tree_path, tree_iter, signal_list):
+            signal_list.append('row-inserted')
+
+        def on_row_changed(tree_store, tree_path, tree_iter, signal_list):
+            signal_list.append('row-changed')
+
+        signals = []
+        tree_store.connect('row-inserted', on_row_inserted, signals)
+        tree_store.connect('row-changed', on_row_changed, signals)
+
+        # adding rows with and without data should only call one signal
+        tree_store.append(None, (0, False))
+        self.assertEqual(signals, ['row-inserted'])
+
+        signals.pop()
+        tree_store.append(None)
+        self.assertEqual(signals, ['row-inserted'])
+
+        signals.pop()
+        tree_store.prepend(None, (0, False))
+        self.assertEqual(signals, ['row-inserted'])
+
+        signals.pop()
+        tree_store.prepend(None)
+        self.assertEqual(signals, ['row-inserted'])
+
+        signals.pop()
+        tree_store.insert(None, 1, (0, False))
+        self.assertEqual(signals, ['row-inserted'])
+
+        signals.pop()
+        tree_store.insert(None, 1)
+        self.assertEqual(signals, ['row-inserted'])
+
     def test_list_store(self):
         class TestPyObject(object):
             pass
@@ -1171,6 +1229,43 @@ class TestGtk(unittest.TestCase):
 
         self.assertEquals(i, 102)
 
+    def test_list_store_signals(self):
+        list_store = Gtk.ListStore(int, bool)
+
+        def on_row_inserted(list_store, tree_path, tree_iter, signal_list):
+            signal_list.append('row-inserted')
+
+        def on_row_changed(list_store, tree_path, tree_iter, signal_list):
+            signal_list.append('row-changed')
+
+        signals = []
+        list_store.connect('row-inserted', on_row_inserted, signals)
+        list_store.connect('row-changed', on_row_changed, signals)
+
+        # adding rows with and without data should only call one signal
+        list_store.append((0, False))
+        self.assertEqual(signals, ['row-inserted'])
+
+        signals.pop()
+        list_store.append()
+        self.assertEqual(signals, ['row-inserted'])
+
+        signals.pop()
+        list_store.prepend((0, False))
+        self.assertEqual(signals, ['row-inserted'])
+
+        signals.pop()
+        list_store.prepend()
+        self.assertEqual(signals, ['row-inserted'])
+
+        signals.pop()
+        list_store.insert(1, (0, False))
+        self.assertEqual(signals, ['row-inserted'])
+
+        signals.pop()
+        list_store.insert(1)
+        self.assertEqual(signals, ['row-inserted'])
+
     def test_tree_path(self):
         p1 = Gtk.TreePath()
         p2 = Gtk.TreePath.new_first()
@@ -1195,6 +1290,8 @@ class TestGtk(unittest.TestCase):
         self.assertFalse(p1 < None)
         self.assertFalse(p1 <= None)
 
+        self.assertEquals(tuple(p1), (1, 2, 3))
+
     def test_tree_model(self):
         tree_store = Gtk.TreeStore(int, str)
 
@@ -1412,6 +1509,19 @@ class TestGtk(unittest.TestCase):
 
         self.assertRaises(ValueError, set_row3)
 
+    def test_tree_view(self):
+        store = Gtk.ListStore(int, str)
+        store.append((0, "foo"))
+        store.append((1, "bar"))
+        view = Gtk.TreeView()
+        # We can't easily call get_cursor() to make sure this works as
+        # expected as we need to realize and focus the column
+        view.set_cursor(store[1].path)
+        view.set_cursor(str(store[1].path))
+
+        view.get_cell_area(store[1].path)
+        view.get_cell_area(str(store[1].path))
+
     def test_tree_view_column(self):
         cell = Gtk.CellRendererText()
         column = Gtk.TreeViewColumn(title='This is just a test',
@@ -1527,15 +1637,27 @@ class TestGtk(unittest.TestCase):
 
         # test Gtk.Button
         button = Gtk.Button()
+        self.failUnless(isinstance(button, Gtk.Button))
+        self.failUnless(isinstance(button, Gtk.Container))
+        self.failUnless(isinstance(button, Gtk.Widget))
         button = Gtk.Button(stock=Gtk.STOCK_CLOSE)
         self.assertEquals(Gtk.STOCK_CLOSE, button.get_label())
         self.assertTrue(button.get_use_stock())
         self.assertTrue(button.get_use_underline())
 
+        # test Gtk.Button use_stock
+        button = Gtk.Button(label=Gtk.STOCK_CLOSE, use_stock=True, use_underline=True)
+        self.assertEquals(Gtk.STOCK_CLOSE, button.get_label())
+        self.assertTrue(button.get_use_stock())
+        self.assertTrue(button.get_use_underline())
+
         # test Gtk.LinkButton
         self.assertRaises(TypeError, Gtk.LinkButton)
-        button = Gtk.LinkButton('http://www.gtk.org', 'Gtk')
-        self.assertEquals('http://www.gtk.org', button.get_uri())
+        button = Gtk.LinkButton('http://www.Gtk.org', 'Gtk')
+        self.failUnless(isinstance(button, Gtk.Button))
+        self.failUnless(isinstance(button, Gtk.Container))
+        self.failUnless(isinstance(button, Gtk.Widget))
+        self.assertEquals('http://www.Gtk.org', button.get_uri())
         self.assertEquals('Gtk', button.get_label())
 
     def test_inheritance(self):
@@ -1578,6 +1700,7 @@ class TestGtk(unittest.TestCase):
 
     def test_label(self):
         label = Gtk.Label(label='Hello')
+        self.failUnless(isinstance(label, Gtk.Widget))
         self.assertEquals(label.get_text(), 'Hello')
 
     def adjustment_check(self, adjustment, value=0.0, lower=0.0, upper=0.0,
@@ -1611,6 +1734,9 @@ class TestGtk(unittest.TestCase):
 
     def test_table(self):
         table = Gtk.Table()
+        self.failUnless(isinstance(table, Gtk.Table))
+        self.failUnless(isinstance(table, Gtk.Container))
+        self.failUnless(isinstance(table, Gtk.Widget))
         self.assertEquals(table.get_size(), (1,1))
         self.assertEquals(table.get_homogeneous(), False)
         table = Gtk.Table(2, 3)
@@ -1629,11 +1755,15 @@ class TestGtk(unittest.TestCase):
         self.assertEquals(table.get_homogeneous(), True)
 
         label = Gtk.Label(label='Hello')
+        self.failUnless(isinstance(label, Gtk.Widget))
         table.attach(label, 0, 1, 0, 1)
         self.assertEquals(label, table.get_children()[0])
 
     def test_scrolledwindow(self):
         sw = Gtk.ScrolledWindow()
+        self.failUnless(isinstance(sw, Gtk.ScrolledWindow))
+        self.failUnless(isinstance(sw, Gtk.Container))
+        self.failUnless(isinstance(sw, Gtk.Widget))
         sb = sw.get_hscrollbar()
         self.assertEquals(sw.get_hadjustment(), sb.get_adjustment())
         sb = sw.get_vscrollbar()
@@ -1678,6 +1808,55 @@ class TestGtk(unittest.TestCase):
         self.assertTrue(hasattr(widget.drag_dest_set_proxy, '__call__'))
         self.assertTrue(hasattr(widget.drag_get_data, '__call__'))
 
+    def test_scrollbar(self):
+        # PyGTK compat
+        adjustment = Gtk.Adjustment()
+
+        hscrollbar = Gtk.HScrollbar()
+        vscrollbar = Gtk.VScrollbar()
+        self.assertNotEquals(hscrollbar.props.adjustment, adjustment)
+        self.assertNotEquals(vscrollbar.props.adjustment, adjustment)
+
+        hscrollbar = Gtk.HScrollbar(adjustment)
+        vscrollbar = Gtk.VScrollbar(adjustment)
+        self.assertEquals(hscrollbar.props.adjustment, adjustment)
+        self.assertEquals(vscrollbar.props.adjustment, adjustment)
+
+    def test_iconview(self):
+        # PyGTK compat
+        iconview = Gtk.IconView()
+        self.assertEquals(iconview.props.model, None)
+
+        model = Gtk.ListStore(str)
+        iconview = Gtk.IconView(model)
+        self.assertEquals(iconview.props.model, model)
+
+    def test_toolbutton(self):
+        # PyGTK compat
+        button = Gtk.ToolButton()
+        self.assertEquals(button.props.stock_id, None)
+
+        button = Gtk.ToolButton('gtk-new')
+        self.assertEquals(button.props.stock_id, 'gtk-new')
+
+    def test_iconset(self):
+        # PyGTK compat
+        iconset = Gtk.IconSet()
+        pixbuf = GdkPixbuf.Pixbuf()
+        iconset = Gtk.IconSet(pixbuf)
+
+    def test_viewport(self):
+        # PyGTK compat
+        vadjustment = Gtk.Adjustment()
+        hadjustment = Gtk.Adjustment()
+
+        viewport = Gtk.Viewport(hadjustment=hadjustment,
+                                vadjustment=vadjustment)
+
+        self.assertEquals(viewport.props.vadjustment, vadjustment)
+        self.assertEquals(viewport.props.hadjustment, hadjustment)
+
+
 class TestGio(unittest.TestCase):
     def setUp(self):
         os.environ['GSETTINGS_BACKEND'] = 'memory'
